#property  indicator_separate_window
#property  indicator_buffers 2
#property  indicator_color1  clrLimeGreen
#property  indicator_color2  clrDarkOrange
#property  indicator_width1  2
#property  indicator_width2  2
#property  indicator_minimum 0
#property  indicator_maximum 1

extern int                FastEma          = 12;
extern int                SlowEma          = 26;
extern int                SignalEma        = 9;
extern ENUM_APPLIED_PRICE Price            = PRICE_CLOSE;
extern bool               alertsOn         = true;
extern bool               alertsOnCurrent  = false;
extern bool               alertsMessage    = true;
extern bool               alertsSound      = true;
extern bool               alertsNotify     = false;
extern bool               alertsEmail      = false;
extern string             soundFile        = "alert2.wav";
extern bool               arrowsVisible    = false;             // Arrows visible?
extern string             arrowsIdentifier = "macd Arrows1";    // Unique ID for arrows
extern double             arrowsUpperGap   = 0.5;               // Upper arrow gap
extern double             arrowsLowerGap   = 0.5;               // Lower arrow gap
extern color              arrowsUpColor    = clrLimeGreen;
extern color              arrowsDnColor    = clrRed;
extern int                arrowsUpCode     = 241;
extern int                arrowsDnCode     = 242;
extern bool               ShowLines        = false;
extern string             linesIdentifier  = "macd lines1";
extern color              linesUpColor     = clrDeepSkyBlue;
extern color              linesDnColor     = clrPaleVioletRed;
extern ENUM_LINE_STYLE    LinesStyle       = STYLE_DOT;
extern int                LinesWidth       = 0;

double signal[];
double barup[];
double bardn[];
double trend[];

//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,barup);  SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,bardn);  SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,signal); 
   SetIndexBuffer(3,trend); 
   return(0);
}
int deinit()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   deleteLines();
return(0);
}
int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         
   //
   //
   //
   //
   //
   
   double alpha = 2.0/(SignalEma+1);
   for (int i=limit; i>=0; i--)   
   {
      double macd  = iMA(NULL,0,FastEma,0,MODE_EMA,Price,i)-iMA(NULL,0,SlowEma,0,MODE_EMA,Price,i);
         signal[i] = signal[i+1]+alpha*(macd-signal[i+1]);
         barup[i]  = EMPTY_VALUE;
         bardn[i]  = EMPTY_VALUE;
         trend[i]  = trend[i+1];
            if (signal[i] < macd) trend[i] = 1; 
            if (signal[i] > macd) trend[i] =-1; 
            if (trend[i] == 1)    barup[i] = 1;
            if (trend[i] ==-1)    bardn[i] = 1;
            
            //
            //
            //
            //
            //
              
            if (arrowsVisible)
            {
               string lookFor = arrowsIdentifier+":"+(string)Time[i]; ObjectDelete(lookFor);  
               if (trend[i]!= trend[i+1])
               {
                 if (trend[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,false);
                 if (trend[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode, true);
               }
             }          
            
            //
            //
            //
            //
            //
             
            if (ShowLines)
            {
               deleteLine(Time[i]);
               if (trend[i]!= trend[i+1])
               {
                  if (trend[i] == 1) drawLine(i,linesUpColor);
                  if (trend[i] ==-1) drawLine(i,linesDnColor);
               }
             }
   }
   
   //
   //
   //
   //
   //
   
   if (alertsOn)
   {
     if (alertsOnCurrent)
          int whichBar = 0;
     else     whichBar = 1;
         
     if (trend[whichBar] != trend[whichBar+1])
     if (trend[whichBar] == 1)
           doAlert("crossing signal up");
     else  doAlert("crossing signal down");       
   }
   return(0);
}

//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Macd ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Macd "),message);
             if (alertsNotify)  SendNotification(message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
  
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//
//
//
//
//

void drawLine(int i,color theColor)
{
     string name = linesIdentifier+":"+Time[i];
   
     //
     //
     //
     //
     //
      
     ObjectCreate(name,OBJ_VLINE,0,Time[i],0);
        ObjectSet(name,OBJPROP_COLOR,theColor);
        ObjectSet(name,OBJPROP_STYLE,LinesStyle);
        ObjectSet(name,OBJPROP_WIDTH,LinesWidth);
        ObjectSet(name,OBJPROP_BACK,true);
}

//
//
//
//
//

void deleteLines()
{
   string lookFor       = linesIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   
   //
   //
   //
   //
   //
   
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
      if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteLine(datetime time)
{
    string lookFor = linesIdentifier+":"+time; ObjectDelete(lookFor);
}


